@@ -1,15 +1,23 @@
 package Devel::IPerl::Kernel;
-$Devel::IPerl::Kernel::VERSION = '0.008';
+$Devel::IPerl::Kernel::VERSION = '0.009';
 use strict;
 use warnings;
 use namespace::autoclean;
 
 use Moo;
+use Env qw(@PATH);
+use if $^O eq 'MSWin32', 'Alien::ZMQ::latest';
+
+BEGIN {
+	if ( $^O eq 'MSWin32' ) {
+		unshift @PATH, Alien::ZMQ::latest->bin_dir;
+	}
+}
+
 
 use ZMQ::LibZMQ3;
 use ZMQ::Constants
 	qw( ZMQ_PUB ZMQ_REP ZMQ_ROUTER
-	    ZMQ_FD
 	    ZMQ_RCVMORE ZMQ_SNDMORE
 	    ZMQ_FORWARDER );
 use JSON::MaybeXS;
@@ -17,6 +25,9 @@ use Path::Class;
 use IO::Async::Loop;
 use IO::Async::Handle;
 use IO::Handle;
+use IO::Async::Routine;
+use Net::Async::ZMQ;
+use Net::Async::ZMQ::Socket;
 use Devel::IPerl::Kernel::Callback::REPL;
 use Devel::IPerl::Message::ZMQ;
 
@@ -167,7 +178,6 @@ sub _assign_ports_from_data {
 #}}}
 #}}}
 
-my @io_handles;
 sub run {#{{{
 	my ($self) = @_;
 	STDOUT->autoflush(1);
@@ -178,13 +188,14 @@ sub run {#{{{
 
 	$self->_setup_heartbeat;
 
+	my $zmq = Net::Async::ZMQ->new;
+
 	my @socket_funcs = ( \&shell, \&control, \&stdin, \&iopub );
 	for my $socket_fn (@socket_funcs) {
 		my $socket = $self->$socket_fn();
-		my $socket_fd = zmq_getsockopt( $socket, ZMQ_FD );
-		my $io_handle = IO::Handle->new_from_fd( $socket_fd, 'r' );
-		my $handle =  IO::Async::Handle->new(
-			handle => $io_handle,
+
+		my $async_socket =  Net::Async::ZMQ::Socket->new(
+			socket => $socket,
 			on_read_ready => sub {
 				my @blobs;
 				while ( my $recvmsg = zmq_recvmsg( $socket, ZMQ_RCVMORE ) ) {
@@ -196,16 +207,19 @@ sub run {#{{{
 					$self->route_message(\@blobs, $socket);
 				}
 			},
-			on_write_ready => sub { },
 		);
-		$self->_loop->add( $handle );
+
+		$zmq->add_child( $async_socket );
 	}
+
+	$self->_loop->add( $zmq );
+
 	$self->_loop->loop_forever;
 }
 
 sub stop {
 	my ($self) = @_;
-	kill 1, $self->_heartbeat_child;
+	$self->_heartbeat_child->kill(1);
 
 	# TODO find out why this gives the error
 	# "Bad file descriptor (epoll.cpp:67)"
@@ -243,15 +257,16 @@ sub kernel_exit {
 sub _setup_heartbeat {
 	my ($self) = @_;
 	# heartbeat socket is just an echo server
-	my $child = $self->_loop->spawn_child(
+	my $child = IO::Async::Routine->new(
 		code => sub {
 			$self->clear_zmq; # need to create new context for this process
 			zmq_device( ZMQ_FORWARDER, $self->heartbeat, $self->heartbeat );
 		},
-		on_exit => sub {
+		on_return => sub {
 			$self->kernel_exit;
 		},
 	);
+	$self->_loop->add( $child );
 	$SIG{INT} = sub { $self->kernel_exit };
 	$self->_heartbeat_child( $child );
 }
@@ -274,7 +289,7 @@ Devel::IPerl::Kernel
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 AUTHOR
 
